﻿package tv.snorkl.display.interactive {
	import flash.display.*;
	import com.greensock.*;
	import com.greensock.plugins.*;
	import com.greensock.easing.*;
	import flash.events.MouseEvent;

	public class GrowAndColorizeItem extends Sprite {
		private var _img:DisplayObject;
		private var _maskWidth:int;
		private var _maskHeight:int;
		private var _container:Sprite;
		private var _mask:Sprite;
		private var _cover:Sprite;//additional Sprite that covers the mask and image to get around eratic rollover/out behavior
		private var _fullWidth:Number;
		private var _fullHeight:Number;
		private var _parent:DisplayObjectContainer;
		private var _tween:TweenMax;

		public function GrowAndColorizeItem($img:DisplayObject, $width:int=0, $height:int=0){

			super();
			TweenPlugin.activate([ColorMatrixFilterPlugin]);
			_img = $img;

			//figure out where on the display list the object should be placed / nested
			_parent = _img.parent;

			this.x = _img.x;
			this.y = _img.y;

			if ($width == 0 || $height == 0) {
				//normal size
				_maskWidth = _img.width;
				_maskHeight = _img.height;
				_fullWidth = _img.width * 1.2;
				_fullHeight = _img.height * 1.2;

			}
			else {
				//custom size
				_maskWidth = $width;
				_maskHeight = $height;
				_fullWidth = _img.width;
				_fullHeight = _img.height;
			}
			build();
		}

		private function build():void {

			//create _container which will be center inside the object
			_container = new Sprite();
			_container.x = _maskWidth * .5;
			_container.y = _maskHeight * .5;
			addChild(_container);

			//center _img inside _container
			_img.x = _img.width * -.5;
			_img.y = _img.height * -.5;

			//if the source is on the display list, figure out where it is nested and place this object there
			if (_parent) {
				_parent.addChildAt(this, _parent.getChildIndex(_img));
			}

			_container.addChild(_img);
			_container.width = _maskWidth;
			_container.height = _maskHeight;

			_mask = createRectangle(_maskWidth,_maskHeight);
			addChild(_mask);
			_container.mask = _mask;
			TweenMax.to(_container, 0, {colorMatrixFilter:{saturation:0, brightness:.8}});
			_tween = TweenMax.to(_container,.3,{width:_fullWidth,height:_fullHeight,colorMatrixFilter:{saturation:1.2,brightness:1},paused:true});

			//_cover is a transparent Sprite that will recieve mouse events (this is a workound for odd mouse behavior on items with ColorMatrixFilter applied)
			//colorMatrixFilter seems to effect the hitarea of DisplayObjects and causes unexpected MouseEvents to fire
			_cover = createRectangle(_maskWidth,_maskHeight,0);
			_cover.addEventListener(MouseEvent.ROLL_OVER, mouseHandler);
			_cover.addEventListener(MouseEvent.ROLL_OUT, mouseHandler);
			_cover.buttonMode = true;
			addChild(_cover);
		}

		private function mouseHandler(e:MouseEvent):void {
			switch (e.type) {
				case "rollOver" :
					_tween.play();
					break;

				case "rollOut" :
					_tween.reverse();
					break;
			}
		}



		private function createRectangle(width:Number, height:Number, alpha:Number = 1, color:Number = 0xff0000):Sprite {
			var rect:Sprite = new Sprite();
			rect.graphics.beginFill(color, alpha);
			rect.graphics.drawRect(0, 0, width,height);
			rect.graphics.endFill();
			return rect;
		}

	}

}